/** ------------------------------------------------------------------------------------
    iframe_guest.js 

    Copyright 2022 Kirk Siqveland (Kirk@CyberTamer.com)

    This file is distributed under the terms of the GNU General Public 
    License as published by the Free Software Foundation; either 
    version 2 of the License.     

    This code was developed to work with the PMWiki wiki framework, 
    however, it also works independently of the PMWiki environment.   

    USAGE:
    for non-pmwiki sites:
    >> You will need a reference in your html to the iframe_host.js 
    >> iframe_host.js and iframe_guest.js must be in the same directory.
    >> the <iframe> tag in your html needs:
    >> class="BridgeFrame" and a sequential number from 0 for id  e.g. id="0" 

    >> in PMWiki you need the BridgeSite.php file in your {pmwiki}/cookbook directory
    >> you will also need the following lines into your {pmwiki}/local/config.php file:

        ## Include Code for BridgeSite - iFrame insertion tool recipe
            include_once ('cookbook/BridgeSite.php') ;
        ## Insert the iframe_host.js script to the bottom of your Skin template.
            $HTMLFooterFmt['BridgeSite'] = '<script src="$FarmPubDirUrl/iframe_host.js"></script>' ;

    >> See https://www.pmwiki.org/wiki/Cookbook/BridgeSite

/** -------------------------------------------------------------------------------------**/
/** -------------------------------------------------------------------------------------**/

    /** Timer Loop Rate  **/    
        TimerLoopRate = 400 ;

    /** Make sure this Guest will fit properly in the Host iFrame  **/
        document.body.style.margin   = "0"       ;
        document.body.style.overflow = "hidden"  ; 

    /** Get page id from host-iframe attribute "hfid"   (assigned in the host page ) **/
        var MyID =  window.frameElement.getAttribute("hfid") ;  

    /** The most dependable way to catch size changes: run a timer and check.  **/
        var resizeTimer     ;  // Establish a timer to use later
        var loadingTimer    ;  // Establish a timer to use later
        var pegHeight   = 0 ;  // Remember sizing changes to reduce cycle-time

/** -- Send messages to the parent/host window --  **/
    function sendMessage(TxMsg) {
        window.parent.postMessage(TxMsg, '*') ;
    }

/** -- Notify Host Only Once page is loaded **/
    function loadTimer(){
        clearTimeout(loadingTimer) ;
        if(MyID === null||MyID === NaN){
            loadingTimer = setTimeout(loadTimer, TimerLoopRate) ;
        }else{
            sizeTest() ;
            sendMessage("guestLoaded,"+MyID+",") ;
            var gotHeight = document.body.offsetHeight; 
            sendMessage("SizeMe," + gotHeight + "," + MyID + ",") ; 
            // loadingTimer = setTimeout(loadTimer, 3000) ;
        }
    }        

/** -- Check Current size against save size --  **/
    function sizeTest(){
        try {
            var gotHeight = document.body.offsetHeight;            
        }
        catch{
            resizeTimer = setTimeout(sizeTest, TimerLoopRate) ;
            return;
        }

        if (gotHeight != pegHeight){   
            /** Save the new size as a benchmark (peg)  **/
                pegHeight = gotHeight ;
            /** inform our Host of the change in size:  **/     
                sendMessage("SizeMe," + gotHeight + "," + MyID + ",") ; 
        }  
        resizeTimer = setTimeout(sizeTest, TimerLoopRate) ;
    }

/** Listen for messages from parent window  **/
    document.addEventListener('message',  function (RxMsg) {
        console.log ("Guest - got message: "+ RxMsg.data + " MyID:"+MyID) ; 
        msg = RxMsg.data.split(",") ;

        /** switch for easier development/mod  **/
        switch(msg[0]) {
              // case "ping":
              //   sendMessage("echo, pinged:" + msg[1] + " --  Respond:" + MyID + "#") ; 
              //   break ;

              case "reset":
                sizeMsg(0) ;
                break ;

              case "ID":
                console.log("Guest rx ID Message: " + msg[1]);
                // MyID = msg[1] ;
                // sizeMsg(0)    ;   
                break ;

              default:
                //console.info("Message Received = "+ RxMsg.data);
        }

        console.log ("Guest - got message: "+ RxMsg.data + " MyID:"+MyID);
            
    }, false) ;

/** -- RESPOND to RESIZE events **/
    window.addEventListener('resize', sizeTest, false);

/* Run the Timer-tests: */
loadTimer() ;
sizeTest()  ;


