/** ---------------------------------------------------------------------------------------------
    iframe_host.js 

    Copyright 2022 Kirk Siqveland (Kirk@CyberTamer.com)

    This file is distributed under the terms of the GNU General Public 
    License as published by the Free Software Foundation; either 
    version 2 of the License.     

    This code was developed to work with the PMWiki wiki framework, 
    however, it also works independently of the PMWiki environment.   

    USAGE:
    for non-pmwiki sites:
    >> You will need a reference in your html to the iframe_host.js 
    >> iframe_host.js and iframe_guest.js must be in the same directory.
    >> the <iframe> tag in your html needs:
    >> class="BridgeFrame" and a sequential number from 0 for id  e.g. id="0" 

    >> in PMWiki you need the BridgeSite.php file in your {pmwiki}/cookbook directory
    >> you will also need the following lines into your {pmwiki}/local/config.php file:

        ## Include Code for BridgeSite - iFrame insertion tool recipe
            include_once ('cookbook/BridgeSite.php') ;
        ## Insert the iframe_host.js script to the bottom of your Skin template.
            $HTMLFooterFmt['BridgeSite'] = '<script src="$FarmPubDirUrl/iframe_host.js"></script>' ;

    >> See https://www.pmwiki.org/wiki/Cookbook/BridgeSite
---------------------------------------------------------------------------------------------**/

/** First we make an array if all the BridgeSite Frames, then configure some style settings, **/

        var HostFrames  = document.getElementsByClassName('BridgeFrame') ;
        var FrameTally  = HostFrames.length ;
        var Injected    = new Array() ;
        var Tally       = 0 ;    
        var loadTimer       ;
        var pegLoaded   = false;

    /** For each instance of a BridgeFrame we set an ID value and styles... 
        Included here to ensure things work better from the start, but you might
        want to remove the .style settings and handle it in your CSS files                  **/
        for (var tick = 0; tick < FrameTally; tick++) {
            HostFrames[tick].setAttribute("hfid", tick)  ;            
            HostFrames[tick].style.width      = "100%"   ;
            HostFrames[tick].style.margin     = "0"      ;
            HostFrames[tick].style.padding    = "0"      ;
            HostFrames[tick].style.border     = "none"   ;
            // HostFrames[tick].style.overflow   = "hidden" ;
        }

    /** -- Listen for messages from hosted iframe-page(s)  **/
        window.addEventListener('message', function (RxMsg) {
            /** -- Parse Received Message - comma delimited  **/
                msg = RxMsg.data.split(",");
                /**  first number is height, second is array position **/
                /**  defaults to 0 if none given. **/
            
            switch(msg[0]) {
                case "SizeMe":
                    if (Number(msg[1]) > 0){
                        var HFid = msg[2]||0 ;
                        HostFrames[HFid].style.height = msg[1] + "px" ;
                    }
                    break;

                case "guestLoaded":
                    /** Debugging or Logging option  **/
                    console.log("RX guestLoaded: "+msg[1]) ;
                    break ;  

                // case "echo":
                //     console.log("Echo recieved: "+msg[1]) ;
                //     break;

                default:
                    console.log("Rx message unkown: "+RxMsg.data);
            }                 
        }, false);

/** ----------------------------------------------------------------------------
        This Injects the iframe_guest.js file into the guest webpage
       ( Only works when guest has the same domain & port & protocol as the host. )
    ----------------------------------------------------------------------------**/

    /** Prepare the path for the injected iframe_guest.js file              **/
    /** getting the url for the current script (this file):                 **/
        var scripts     = document.getElementsByTagName("script")         ;
        /** This should be the latest script loaded, so:    **/
        var ScriptSRC   = scripts[scripts.length-1].src ;  //Script file path

    /** Swap in the iframe_guest.js filename, since they should be in the same place **/
        ScriptSRC           = ScriptSRC.replace("iframe_host","iframe_guest") ;
        console.log("ScriptSRC = "+ScriptSRC);
        if(typeof(ScriptSRC) == "undefined" || ScriptSRC == null || ScriptSRC == "") {
            ScriptSRC = document.currentScript.src ; }


    function scriptLoader() { 
        clearTimeout(loadTimer) ;
        Tally = 0 ;

        for (   var tick = 0; tick < FrameTally; tick++) {
            /** Tally all the guest already loaded (testing for title) **/

            var guestDoc  = HostFrames[tick].contentDocument || 
                            HostFrames[tick].contentWindow ;     
            if(guestDoc.document) guestDoc = guestDoc.document;  

            // if(guestDoc !== null && guestDoc.title !== null){   
            if(guestDoc.title !== ""){ 
                Tally++;
            }         
        }

        if( Tally < FrameTally ){ 
          /** If not all frames are loaded... wait and try again. **/
            // console.log("scriptLoader - Tally = "+Tally);
            loadTimer = window.setTimeout(scriptLoader, 120); 
            return;
            // } else{
            //     console.log("scriptLoader - iFrames All Loaded!");
        }        


        for (   var tick = 0; tick < FrameTally; tick++) {  
            // console.log("scriptLoader - attempting to inject iFrame "+tick);


            var guestDoc  = HostFrames[tick].contentDocument || 
                            HostFrames[tick].contentWindow ;     
            if (guestDoc.document) guestDoc = guestDoc.document; 
             
            if(Injected[tick]=== undefined || Injected[tick] !== "done"){
                /** Test if guest iFrame includes iframe_guest script already  **/
                if(guestDoc.body.innerHTML.includes("iframe_guest")){
                    Injected[tick] = "done" ;
                }else{
                    if (guestDoc.documentElement) guestDoc = guestDoc.documentElement;
                
                    /** Establish the <body> element so we can add the script as the last child **/
                    var guestBodies = guestDoc.getElementsByTagName('body');
                    if(guestBodies[0]!==undefined){
                        var guestBody   = guestBodies[0];                    
                        /**  Build and inject a script element with the iframe_guest.js **/
                        /**  Which must be in the same directory as the iframe_host.js  **/
                            const newNode     = document.createElement("script");
                                  newNode.src = ScriptSRC  ;   
                                  newNode.setAttribute('id', 'iframe_guest');  

                            try{  guestBody.appendChild(newNode) ; }
                                catch(err) {
                                    loadTimer = window.setTimeout(scriptLoader, 500);
                                    console.log(" append Sript Error: err.message = " + err.message);
                                    return;
                                }
                            console.log("scriptLoader - Success injecting iFrame "+tick);
                            /** Flag completed injection  **/
                            Injected[tick] = "done" ;
                    }
                }
            }
        }  
    } /** end of scriptLoader()  **/


    /** Run the scriptLoader once this file is loaded... **/
        scriptLoader() ;